/*
Liquid War 6 is a unique multiplayer wargame.
Copyright (C)  2005, 2006  Christian Mauduit <ufoot@ufoot.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
Contact author        : ufoot@ufoot.org
*/

#include <string.h>

#include "config.h"
#include "../../gfx.h"
#include "gl-utils.h"
#include "gl-utils-internal.h"

/*
 * Creates a unique string key for a given menu.
 * Pointer must be freed.
 */
static char *
create_button_key (char *text, int select)
{
  char *key;
  int length;

  length = strlen (text) + 1 + 33;	// ':' + maxlength("%d")
  key = (char *) LW6SYS_MALLOC ((length + 1) * sizeof (char));
  if (key != NULL)
    {
      snprintf (key, length + 1, "%d:%s", select, text);
      key[length] = '\0';
    }

  return key;
}

/*
 * Store a button bitmap in the menucache. Returns != 0 if successfull.
 */
int
mod_gl_utils_store_button_in_menucache (MOD_GL_UTILS_CONTEXT * context,
					char *text, int select,
					SDL_Surface * surface)
{
  MOD_GL_UTILS_MENUCACHE_ARRAY *menucache_array;
  int ret = 0;
  int i;
  char *key;
  char *old_key;
  SDL_Surface *old_surface;

  menucache_array = &(context->menucache_array);

  i = menucache_array->last_stored_item =
    (menucache_array->last_stored_item +
     1) % MOD_GL_UTILS_MENUCACHE_ARRAY_SIZE;

  if ((old_key = menucache_array->item_array[i].key) != NULL)
    {
      LW6SYS_FREE (old_key);
      menucache_array->item_array[i].key = NULL;
    }
  if ((old_surface = menucache_array->item_array[i].surface) != NULL)
    {
      mod_gl_utils_delete_surface (context, old_surface);
      menucache_array->item_array[i].surface = NULL;
    }

  key = create_button_key (text, select);
  if (key != NULL)
    {
      ret = 1;
      menucache_array->item_array[i].key = key;
      menucache_array->item_array[i].surface = surface;
    }

  return ret;
}

/*
 * Gets a button bitmap from the menucache. Returns non NULL if successfull.
 */
SDL_Surface *
mod_gl_utils_get_button_from_menucache (MOD_GL_UTILS_CONTEXT * context,
					char *text, int select)
{
  MOD_GL_UTILS_MENUCACHE_ARRAY *menucache_array;
  SDL_Surface *surface = NULL;
  int i;
  char *key;

  menucache_array = &(context->menucache_array);

  key = create_button_key (text, select);

  if (key != NULL)
    {
      for (i = 0; i < MOD_GL_UTILS_MENUCACHE_ARRAY_SIZE; ++i)
	{
	  if (menucache_array->item_array[i].key != NULL &&
	      menucache_array->item_array[i].surface != NULL)
	    {
	      if (strcmp (key, menucache_array->item_array[i].key) == 0)
		{
		  surface = menucache_array->item_array[i].surface;
		  break;
		}
	    }
	}
      LW6SYS_FREE (key);
    }

  return surface;
}

/*
 * Clear the cache (usefull when quiting the program).
 */
void
mod_gl_utils_clear_menucache (MOD_GL_UTILS_CONTEXT * context)
{
  MOD_GL_UTILS_MENUCACHE_ARRAY *menucache_array;
  int i;
  char *key;
  SDL_Surface *surface;

  menucache_array = &(context->menucache_array);

  for (i = 0; i < MOD_GL_UTILS_MENUCACHE_ARRAY_SIZE; ++i)
    {
      if ((key = menucache_array->item_array[i].key) != NULL)
	{
	  LW6SYS_FREE (key);
	}
      if ((surface = menucache_array->item_array[i].surface) != NULL)
	{
	  mod_gl_utils_delete_surface (context, surface);
	}
    }

  memset (menucache_array, 0, sizeof (MOD_GL_UTILS_MENUCACHE_ARRAY));
}
