/*
Liquid War 6 is a unique multiplayer wargame.
Copyright (C)  2005, 2006  Christian Mauduit <ufoot@ufoot.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
Contact author        : ufoot@ufoot.org
*/

#include <string.h>

#include "config.h"
#include "ker.h"
#include "ker-internal.h"

void
lw6ker_cursor_array_clear (LW6KER_CURSOR_ARRAY * cursor_array)
{
  memset (cursor_array, 0, sizeof (LW6KER_CURSOR_ARRAY));
}

LW6SYS_BOOL
lw6ker_cursor_array_add (LW6KER_CURSOR_ARRAY * cursor_array,
			 LW6KER_CURSOR * cursor)
{
  LW6SYS_BOOL ret = 0;

  if (cursor_array->nb_cursors < LW6OPT_MAX_CURSORS_PER_TEAM)
    {
      cursor_array->cursors[cursor_array->nb_cursors] = *cursor;
      cursor_array->nb_cursors++;
      ret = 1;
    }
  else
    {
      lw6sys_log (LW6SYS_LOG_WARNING, "ker",
		  _("can't add cursor, already %d cursors in cursor_array"),
		  cursor_array->nb_cursors);
    }

  return ret;
}

LW6SYS_BOOL
lw6ker_cursor_array_set (LW6KER_CURSOR_ARRAY * cursor_array, LW6SYS_INT32 i,
			 LW6SYS_INT32 x, LW6SYS_INT32 y,
			 LW6SYS_INT32 pot_offset,
			 LW6KER_MAP_STRUCT * map_struct,
			 LW6SYS_INT32 max_cursor_pot_offset)
{
  LW6SYS_BOOL ret = 0;

  if (i >= 0 && i < LW6OPT_MAX_CURSORS_PER_TEAM)
    {
      if (i >= cursor_array->nb_cursors)
	{
	  lw6ker_cursor_array_force_nb_cursors (cursor_array, i + 1);
	}
      ret =
	lw6ker_cursor_set (&(cursor_array->cursors[i]), x, y, pot_offset,
			   map_struct, max_cursor_pot_offset);
    }

  return ret;
}

extern LW6SYS_BOOL
lw6ker_cursor_array_force_nb_cursors (LW6KER_CURSOR_ARRAY * cursor_array,
				      LW6SYS_INT32 nb_cursors)
{
  LW6SYS_BOOL ret = 0;
  LW6SYS_INT32 i;

  if (nb_cursors >= 0 && nb_cursors < LW6OPT_MAX_CURSORS_PER_TEAM)
    {
      if (nb_cursors > cursor_array->nb_cursors)
	{
	  for (i = cursor_array->nb_cursors; i < nb_cursors; ++i)
	    {
	      lw6ker_cursor_clear (&(cursor_array->cursors[i]));
	    }
	}
      cursor_array->nb_cursors = nb_cursors;
      ret = 1;
    }

  return ret;
}

void
_lw6ker_cursor_array_update_checksum (LW6KER_CURSOR_ARRAY * cursor_array,
				      LW6SYS_UINT32 * checksum)
{
  LW6SYS_INT32 i;

  for (i = 0; i < cursor_array->nb_cursors; ++i)
    {
      _lw6ker_cursor_update_checksum (&(cursor_array->cursors[i]), checksum);
    }
}

void
_lw6ker_cursor_array_activate (LW6KER_CURSOR_ARRAY * cursor_array,
			       LW6SYS_INT32 nb_cursors, LW6SYS_XY pos)
{
  LW6SYS_INT32 i;

  if (nb_cursors >= 0 && nb_cursors < LW6OPT_MAX_CURSORS_PER_TEAM)
    {
      cursor_array->nb_cursors = nb_cursors;
      for (i = 0; i < cursor_array->nb_cursors; ++i)
	{
	  _lw6ker_cursor_init (&(cursor_array->cursors[i]), pos);
	}
    }
  else
    {
      lw6sys_log (LW6SYS_LOG_WARNING, "ker",
		  _
		  ("unable to activate cursor_array, uncorrect cursor number (%d)"),
		  nb_cursors);
    }
}
